#import "MiscString.DExtensions.h"

@implementation MiscString(DExtensions)

#define MISC_OCTAL_REGEX "\\\\[0-9][0-9][0-9]*"

// you could need it to read an .addresses file.
- collapseOctalQuotedDigits
{	int spot,i,val;
	for(i=0;(spot=[self spotOfRegex:MISC_OCTAL_REGEX occurrenceNum:i]) >=0;)
	{	sscanf(buffer+spot+1,"%o",&val);
		if(!val) i++;
		[self replaceRegex:MISC_OCTAL_REGEX withChar:(char)val occurrenceNum:i];
	}
	return self;
}
- collapseBackslashQuotedChars
{	[self replaceEveryOccurrenceOf:"\\n" withChar:'\n'];
	[self replaceEveryOccurrenceOf:"\\t" withChar:'\t'];
	[self replaceEveryOccurrenceOf:"\\r" withChar:'\r'];
	[self replaceEveryOccurrenceOf:"\\\"" withChar:'\"'];
	return self;
}

- expandBackslashQuotedChars
{//	[self replaceEveryOccurrenceOfChar:'\\' with:"\\\\"];
	[self replaceEveryOccurrenceOfChar:'\n' with:"\\n"];
	[self replaceEveryOccurrenceOfChar:'\t' with:"\\t"];
	[self replaceEveryOccurrenceOfChar:'\r' with:"\\r"];
	[self replaceEveryOccurrenceOfChar:'\"' with:"\\\""];
	return self;
}

- expandUnprintableCharsToOctalQuotedDigits
{
#if 0	// not implemented
	while
	if(!NXPrint(c)) theStr=[initFromFormat:"\\%o",c],[self replaceCharWithString:theStr
#endif
	return self;
}

- trimLeadChars:(const char*)chars
{	const char *p=buffer;
	while(strchr(chars,*p)) p++;
	if(p != buffer) [self removeFrom:0 length:p-buffer];
	[self recalcLength];
	return self;
}
- trimTrailChars:(const char*)chars respectQuotationBy:(unsigned int) qc
{	char *p=buffer+strlen(buffer);
	while(strchr(chars,*p)  && (*(MAX(p-1,buffer)) != qc)  ) *p--=0;
	[self recalcLength];
	return self;
}
- trimTrailChars:(const char*)chars
{	return [self trimTrailChars:chars respectQuotationBy:0];
}

-(BOOL) unbalancedQuotation:(char)theChar
{	int cnt=[self numOfChar:theChar];
	if(cnt) return cnt&1;
	else	return NO;
}
- tokenize:(const char *)breakChars into:theList fieldsQuotatedBy:(char) theChar
{	List *tokenList=[self tokenize:breakChars into:theList];
	if(!theList) theList=tokenList;
	if(theChar)
	{	int cnt=0;
		MiscString *theStr;
		while(theStr=[theList objectAt:cnt++])
		{	if([theStr unbalancedQuotation:theChar])
			{	MiscString *currStr;
				while(currStr=[theList objectAt:cnt++])
				{	[theStr cat:breakChars],[theStr concatenate:currStr];
					[theList removeObject:currStr],cnt--;
					if([currStr unbalancedQuotation:theChar])
					{	break;
		}	}	}	}
	} return theList;
}
-(BOOL) streq:(char*)thestr
{	return !strcmp(buffer,thestr);
}

- streamPuts:(NXStream *) stream
{	NXPrintf(stream, "%s\n", buffer);
	return self;
}

BOOL isBreakChar(const char c)
{	return !!strchr(" \t\n\r:,.;()/\\?1234567890[]{}!\"$%&=@'`+*#@<>|^~",c);
}
-(int) spotOfBreakCharsStartAt:(int)n
{	int i;
	for(i=n;i < length;i++)
	{	if(isBreakChar(buffer[i])) return i;
	} return -1;
}
- wordStart:(int *)start andLen:(int *)len ofNum:(int) num
{	int i = 0;
	int currword = 0;
	int spot = 0; 
	int spot2 = 0;

	if (!buffer) return nil;
	while ((i < length) && (currword <= num))
	{	while ((i < length) && (isBreakChar(buffer[i]))) i++;
		spot2 = i;
		while ((i < length) && (!isBreakChar(buffer[i]))) i++;
		spot = i;
		currword++;
	}
	
	if ((spot == spot2) || (num != (currword-1))) return nil;
	*start=spot2,*len=spot-spot2;
	return self;
}
- wordOfNum:(int)num
{	int start=0,len=0;
	if([self wordStart:&start andLen:&len ofNum:num])
		return [self midFrom:start length:len];
	else return nil;
}
- collapseGermanUmlaut
{	[self replaceEveryOccurrenceOf:"ae" withChar:''];
	[self replaceEveryOccurrenceOf:"oe" withChar:''];
	[self replaceEveryOccurrenceOf:"ue" withChar:''];
	[self replaceEveryOccurrenceOf:"ss" withChar:''];
	return self;
}
- setBytes:(const char *)ptr n:(int)n
{	[self setStringValue:""];
	[self cat:ptr n:n];
	return self;
}

@end
